SystemOrganization addCategory: #'Idiom-Exercise'!

TestCase subclass: #IdiomTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Idiom-Exercise'!

!IdiomTest methodsFor: 'testing' stamp: 'JuanManuelTastzian 4/24/2013 17:06'!
should: aBlock notTakeMoreThan: time

	| millisecondsBeforeRunning millisecondsAfterRunning |
		millisecondsBeforeRunning := Time millisecondClockValue.
		aBlock value.
		millisecondsAfterRunning := Time millisecondClockValue.
		self assert: (millisecondsAfterRunning-millisecondsBeforeRunning) < time.! !

!IdiomTest methodsFor: 'testing' stamp: 'JuanManuelTastzian 4/22/2013 21:46'!
should: aBlock throws: anExceptionClass handle: aHandler

	[ aBlock value.
		self fail ]
			on: anExceptionClass  
			do: aHandler.! !

!IdiomTest methodsFor: 'testing' stamp: 'JuanManuelTastzian 4/24/2013 17:06'!
test01AddingCustomerShouldNotTakeMoreThan50Milliseconds

	| customerBook |
	
	customerBook := CustomerBook  new.
	
	self should: [customerBook addCustomerNamed: 'John Lennon'] notTakeMoreThan: 50.
	
! !

!IdiomTest methodsFor: 'testing' stamp: 'JuanManuelTastzian 4/24/2013 17:06'!
test02RemovingCustomerShouldNotTakeMoreThan100Milliseconds

	| customerBook johnLennon |
	
	customerBook := CustomerBook  new.
	johnLennon := 'John Lennon'.
	
	customerBook addCustomerNamed: johnLennon.
	
	self should: [customerBook removeCustomerNamed: 'John Lennon'] notTakeMoreThan: 100.
	
! !

!IdiomTest methodsFor: 'testing' stamp: 'JuanManuelTastzian 4/22/2013 21:46'!
test03CanNotAddACustomerWithEmptyName 

	| customerBook |
			
	customerBook := CustomerBook new.

	self should: [ customerBook addCustomerNamed: ''.]
		throws: Error 
		handle: [ :anError | 
			self assert: anError messageText = CustomerBook customerCanNotBeEmptyErrorMessage.
			self assert: customerBook isEmpty ]! !

!IdiomTest methodsFor: 'testing' stamp: 'JuanManuelTastzian 4/22/2013 21:47'!
test04CanNotRemoveAndInvalidCustomer
	
	| customerBook johnLennon |
			
	customerBook := CustomerBook new.
	johnLennon := 'John Lennon'.
	customerBook addCustomerNamed: johnLennon.
	
	self should: [ customerBook removeCustomerNamed: 'Paul McCartney'. ]
		throws: NotFound 
		handle: [ :anError | 
			self assert: customerBook numberOfCustomers = 1.
			self assert: (customerBook includesCustomerNamed: johnLennon) ]
! !

Object subclass: #CustomerBook
	instanceVariableNames: 'customers'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Idiom-Exercise'!

!CustomerBook class methodsFor: 'error messages' stamp: 'HernanWilkinson 7/6/2011 17:57'!
customerAlreadyExistsErrorMessage

	^'Customer already exists'! !

!CustomerBook class methodsFor: 'error messages' stamp: 'HernanWilkinson 7/6/2011 17:53'!
customerCanNotBeEmptyErrorMessage

	^'Customer name cannot be empty'! !

!CustomerBook methodsFor: 'testing' stamp: 'JuanManuelTastzian 4/22/2013 20:11'!
action: aBlock for: aName  takesLessThan: aNumber
	"comment stating purpose of messaaCustomerNamed ge"

	| timeBefore timeAfter customer |
	timeBefore  := Time millisecondClockValue.
	customer removeCustomerNamed: aName.
	timeAfter := Time millisecondClockValue.
	
	^ (timeAfter - timeBefore) < aNumber.! !

!CustomerBook methodsFor: 'customer management' stamp: 'HernanWilkinson 7/6/2011 17:42'!
addCustomerNamed: aName

	aName isEmpty ifTrue: [ self signalCustomerNameCannotBeEmpty ].
	(customers includes: aName) ifTrue: [ self signalCustomerAlreadyExists ].
	
	customers add: aName ! !

!CustomerBook methodsFor: 'testing' stamp: 'HernanWilkinson 7/6/2011 17:56'!
includesCustomerNamed: aName

	^customers includes: aName ! !

!CustomerBook methodsFor: 'initialization' stamp: 'HernanWilkinson 7/6/2011 17:42'!
initialize

	super initialize.
	customers := OrderedCollection new! !

!CustomerBook methodsFor: 'testing' stamp: 'HernanWilkinson 7/6/2011 17:48'!
isEmpty
	
	^customers isEmpty  ! !

!CustomerBook methodsFor: 'customer management' stamp: 'HernanWilkinson 7/6/2011 17:56'!
numberOfCustomers
	
	^customers size! !

!CustomerBook methodsFor: 'customer management' stamp: 'HernanWilkinson 7/6/2011 17:44'!
removeCustomerNamed: aName
 
	customers remove: aName ! !

!CustomerBook methodsFor: 'customer management' stamp: 'HernanWilkinson 7/6/2011 17:52'!
signalCustomerAlreadyExists 

	self error: self class customerAlreadyExistsErrorMessage! !

!CustomerBook methodsFor: 'customer management' stamp: 'HernanWilkinson 7/6/2011 17:51'!
signalCustomerNameCannotBeEmpty 

	self error: self class customerCanNotBeEmptyErrorMessage ! !
